@@ -1,13 +1,14 @@
 # -*- coding: utf-8 -*-
 
 ###
-### $Release: 0.6.0 $
-### copyright(c) 2008-2011 kuwata-lab.com all rights reserved.
-### MIT License
+### $Release: 0.7.0 $
+### $Copyright: copyright(c) 2008-2011 kuwata-lab.com all rights reserved. $
+### $License: MIT License $
 ###
 
 import sys, os, re
 import shutil
+import subprocess
 #from glob import glob as _glob
 import kook
 from kook import KookCommandError
@@ -33,14 +34,16 @@ def system_f(command):
     _report_cmd(command)
     if config.noexec:
         return
-    return os.system(command)
+    #return os.system(command)
+    return subprocess.call(command, shell=True)
 
 
 def system(command):
     _report_cmd(command)
     if config.noexec:
         return
-    status = os.system(command)
+    #status = os.system(command)
+    status = subprocess.call(command, shell=True)
     if status != 0:
         #status = status % 255
         raise KookCommandError("command failed (status=%s)." % status)
@@ -246,12 +249,12 @@ def _echo(messages, func, cmd, n=False):
     msgs = _prepare(messages, cmd)
     if config.noexec:
         return
-    write = config.stdout.write
+    write = sys.stdout.write
     for i, msg in enumerate(msgs):
         if i > 0: write(' ')
         write(msg)
     if not n: write("\n")
-    config.stdout.flush()      # requires for 'echo -n'
+    sys.stdout.flush()      # requires for 'echo -n'
 
 
 def store(*filenames):
