@@ -1,9 +1,9 @@
 # -*- coding: utf-8 -*-
 
 ###
-### $Release: 0.6.0 $
-### copyright(c) 2008-2011 kuwata-lab.com all rights reserved.
-### MIT License
+### $Release: 0.7.0 $
+### $Copyright: copyright(c) 2008-2011 kuwata-lab.com all rights reserved. $
+### $License: MIT License $
 ###
 
 import sys as _sys
@@ -21,5 +21,5 @@ compare_contents = True
 cmdopt_parser_class = _utils.CommandOptionParser
 properties_filename = 'Properties.py'
 cookbook_filename   = 'Kookbook.py'
-stdout           = _sys.stdout
-stderr           = _sys.stderr
+#stdout           = _sys.stdout      # obsolete
+#stderr           = _sys.stderr      # obsolete
