@@ -1,19 +1,20 @@
 # -*- coding: utf-8 -*-
 
 ###
-### $Release: 0.6.0 $
-### copyright(c) 2008-2011 kuwata-lab.com all rights reserved.
-### MIT License
+### $Release: 0.7.0 $
+### $Copyright: copyright(c) 2008-2011 kuwata-lab.com all rights reserved. $
+### $License: MIT License $
 ###
 
 import sys, os, re, types
+from types import FunctionType
 from kook import KookRecipeError
 from kook.decorators import RecipeDecorator
 from kook.misc import Category, _debug, _trace
 import kook.utils
 from kook.utils import _is_str, to_list, read_file, resolve_filepath, ArgumentError, has_metachars
 import kook.config as config
-from kook.misc import ConditionalFile
+from kook.misc import ConditionalFile, Category
 
 __all__ = ('Cookbook', 'Recipe', )
 
@@ -204,11 +205,7 @@ class Cookbook(ICookbook):
         code_obj = compile(content, filepath or "(kook)", "exec")
         exec(code_obj, context, context)
         #
-        if context is not self.context:
-            #context['__file__'] = filepath
-            if '__export__' in context:
-                for k in context['__export__']:
-                    self.context[k] = context[k]
+        self._hook_context(context)
         #
         _trace("specific task recipes: %s" % repr(self.specific_task_recipes))
         _trace("generic  task recipes: %s" % repr(self.generic_task_recipes))
@@ -216,6 +213,14 @@ class Cookbook(ICookbook):
         _trace("generic  file recipes: %s" % repr(self.generic_file_recipes))
         return context
 
+    def _hook_context(self, context):
+        ## copy exported values
+        if context is not self.context:
+            #context['__file__'] = filepath
+            if '__export__' in context:
+                for k in context['__export__']:
+                    self.context[k] = context[k]
+
     def __get_default(self):
         return self.context.get('kook_default_product')
 
@@ -292,7 +297,7 @@ class Recipe(object):
 
     __category = None
 
-    def __init__(self, kind=None, product=None, ingreds=None, byprods=None, method=None, desc=None, spices=None):
+    def __init__(self, kind=None, product=None, ingreds=None, byprods=None, method=None, desc=None, spices=None, remotes=None):
         self.kind    = kind
         self.product = product
         self.ingreds = ingreds
@@ -300,6 +305,7 @@ class Recipe(object):
         self.method  = method
         self.desc    = desc
         self.spices  = spices
+        self.remotes = remotes
 
     def __get_kind(self):
         return self.__kind
@@ -356,6 +362,12 @@ class Recipe(object):
         self.__spices = to_list(spices)
     spices = property(__get_spices, __set_spices)
 
+    def __get_remotes(self):
+        return self.__remotes
+    def __set_remotes(self, remotes):
+        self.__remotes = to_list(remotes)
+    remotes = property(__get_remotes, __set_remotes)
+
     def __get_pattern(self):
         return self.__pattern
     pattern = property(__get_pattern)
@@ -396,9 +408,10 @@ class Recipe(object):
                 raise ArgumentError("%r: recipe ingredients should be a list or tuple." % (ingreds,))
         byprods = getattr(method, '_kook_byprods', [])
         spices  = getattr(method, '_kook_spices', [])
+        remotes = getattr(method, '_kook_remotes', [])
         desc    = method.__doc__  ## can be empty string
         if desc is None: desc = _default_descs.get(product)
-        return cls(kind=kind, product=product, ingreds=ingreds, byprods=byprods, method=method, desc=desc, spices=spices)
+        return cls(kind=kind, product=product, ingreds=ingreds, byprods=byprods, method=method, desc=desc, spices=spices, remotes=remotes)
 
     def is_generic(self):
         return self.__pattern is not None
