@@ -1,9 +1,9 @@
 # -*- coding: utf-8 -*-
 
 ###
-### $Release: 0.6.0 $
-### copyright(c) 2008-2011 kuwata-lab.com all rights reserved.
-### MIT License
+### $Release: 0.7.0 $
+### $Copyright: copyright(c) 2008-2011 kuwata-lab.com all rights reserved. $
+### $License: MIT License $
 ###
 
 import sys
@@ -13,6 +13,7 @@ from kook import KookRecipeError
 from kook.utils import flatten, _is_str, ArgumentError, get_funcname
 #from kook.cookbook import Recipe
 Recipe = None         # lazy import to avoid mutual import
+Remote = None         # lazy import
 
 __all__ = ('RecipeDecorator', )
 
@@ -26,9 +27,10 @@ class RecipeDecorator(object):
         return { "recipe":  self.recipe,  "product":  self.product,
                  "ingreds": self.ingreds, "byprods":  self.byprods,
                  "coprods": self.coprods, "priority": self.priority,
-                 "spices":  self.spices,  "cmdopts":  self.cmdopts, }
+                 "spices":  self.spices,  "remotes":  self.remotes,
+                 "cmdopts":  self.cmdopts, }
 
-    def recipe(self, product=None, ingreds=None):
+    def recipe(self, product=None, ingreds=None, remotes=None):
         """decorator to mark function as a recipe.
         ex1.
            @recipe
@@ -64,6 +66,7 @@ class RecipeDecorator(object):
         def deco(f):
             if product:  f._kook_product = product
             if ingreds:  f._kook_ingreds = ingreds
+            if remotes:  f._kook_remotes = list(remotes)
             self.recipe(f)
             return f
         return deco
@@ -114,6 +117,19 @@ class RecipeDecorator(object):
         return deco
 
 
+    def remotes(self, *remote_objects):
+        global Remote
+        if not Remote: from kook.remote import Remote
+        remote_objects = flatten(remote_objects)
+        for obj in remote_objects:
+            if not isinstance(obj, Remote):
+                raise TypeError("@remotes(): Remote object expected but got %r." % (obj,))
+        def deco(f):
+            f._kook_remotes = flatten(remote_objects)
+            return f
+        return deco
+
+
     def cmdopts(self, *names):
         import sys
         sys.stderr.write("[pykook] ERROR: '@cmdopts()' is obsolete. Use '@spices()' instead.\n")
