@@ -1,9 +1,9 @@
 # -*- coding: utf-8 -*-
 
 ###
-### $Release: 0.6.0 $
-### copyright(c) 2008-2011 kuwata-lab.com all rights reserved.
-### MIT License
+### $Release: 0.7.0 $
+### $Copyright: copyright(c) 2008-2011 kuwata-lab.com all rights reserved. $
+### $License: MIT License $
 ###
 
 import sys, os
@@ -44,14 +44,24 @@ Category = None
 
 class _CategoryMetaClass(type):
 
+    def __new__(cls, name, bases, dct):
+        ## convert all instance methods into staticmethods
+        for k in dct.keys():
+            v = dct[k]
+            if isinstance(v, FunctionType):
+                dct[k] = staticmethod(v)
+        return type.__new__(cls, name, bases, dct)
+
     def __init__(cls, name, bases, dct):
         type.__init__(cls, name, bases, dct)
         def modify(prefix, outer_cls, d, depth):
             for k in d:
                 v = d[k]
-                if isinstance(v, FunctionType):
-                    if hasattr(v, '_kook_recipe'):
-                        r = v._kook_recipe
+                #if isinstance(v, FunctionType):
+                if isinstance(v, staticmethod):
+                    fn = v.__get__(0)
+                    if hasattr(fn, '_kook_recipe'):
+                        r = fn._kook_recipe
                         if r.kind == 'task':
                             if r.product == 'default' or r.product == '__index__':
                                 r.product = prefix[:-1]
@@ -89,7 +99,7 @@ class Category(object, metaclass=_CategoryMetaClass):
 
 def _debug(msg, depth=0):
     if config.debug_level >= 1:
-        write = config.stdout.write
+        write = sys.stdout.write
         write(config.debug_prompt)
         if depth: write('+' * depth + ' ')
         write(msg)
@@ -98,7 +108,7 @@ def _debug(msg, depth=0):
 
 def _trace(msg, depth=0):
     if config.debug_level >= 2:
-        write = config.stdout.write
+        write = sys.stdout.write
         write(config.debug_prompt)
         if depth: write('+' * depth + ' ')
         write(msg)
@@ -107,7 +117,7 @@ def _trace(msg, depth=0):
 
 def _report_msg(msg, level=None):
     if not config.quiet:
-        write = config.stdout.write
+        write = sys.stdout.write
         write(config.message_prompt)
         if level: write('*' * level + ' ')
         write(msg)
@@ -116,7 +126,7 @@ def _report_msg(msg, level=None):
 
 def _report_cmd(cmd):
     if not config.quiet:
-        write = config.stdout.write
+        write = sys.stdout.write
         write(config.command_prompt)
         write(cmd)
         if cmd[-1] != "\n": write("\n")
