@@ -1,9 +1,9 @@
 # -*- coding: utf-8 -*-
 
 ###
-### $Release: 0.6.0 $
-### copyright(c) 2008-2011 kuwata-lab.com all rights reserved.
-### MIT License
+### $Release: 0.7.0 $
+### $Copyright: copyright(c) 2008-2011 kuwata-lab.com all rights reserved. $
+### $License: MIT License $
 ###
 
 import sys, os, re, types
@@ -26,6 +26,12 @@ def str2int(s):
         return None
 
 
+def setattrs(obj, **kwargs):
+    for k in kwargs:
+        setattr(obj, k, kwargs[k])
+    return obj
+
+
 if python2:
 
     def read_file(filename, encoding=None):
